@@ -6,8 +6,10 @@ from abc import ABC, abstractmethod
 from contextlib import contextmanager
 from os.path import isabs
 
-from ...config.constants import AppEnvVars
-from ...utils.structures import EnvVars
+from hatch.config.constants import AppEnvVars
+from hatch.env.utils import add_verbosity_flag
+from hatch.project.utils import format_script_commands, parse_script_command
+from hatch.utils.structures import EnvVars
 
 
 class EnvironmentInterface(ABC):
@@ -42,17 +44,17 @@ class EnvironmentInterface(ABC):
     PLUGIN_NAME = ''
     """The name used for selection."""
 
-    def __init__(self, root, metadata, name, config, data_directory, platform, verbosity, app=None):
+    def __init__(self, root, metadata, name, config, matrix_variables, data_directory, platform, verbosity, app=None):
         self.__root = root
         self.__metadata = metadata
         self.__name = name
         self.__config = config
+        self.__matrix_variables = matrix_variables
         self.__data_directory = data_directory
         self.__platform = platform
-        self.verbosity = verbosity
+        self.__verbosity = verbosity
         self.__app = app
-
-        self._context = None
+        self.__context = None
 
         self._system_python = None
         self._env_vars = None
@@ -71,10 +73,14 @@ class EnvironmentInterface(ABC):
         self._pre_install_commands = None
         self._post_install_commands = None
 
+    @property
+    def matrix_variables(self):
+        return self.__matrix_variables
+
     @property
     def app(self):
         """
-        An instance of [Application](utilities.md#hatchling.bridge.app.Application).
+        An instance of [Application](../utilities.md#hatchling.bridge.app.Application).
         """
         if self.__app is None:
             from hatchling.bridge.app import Application
@@ -83,6 +89,17 @@ class EnvironmentInterface(ABC):
 
         return self.__app
 
+    @property
+    def context(self):
+        if self.__context is None:
+            self.__context = self.get_context()
+
+        return self.__context
+
+    @property
+    def verbosity(self):
+        return self.__verbosity
+
     @property
     def root(self):
         """
@@ -104,14 +121,14 @@ class EnvironmentInterface(ABC):
     @property
     def platform(self):
         """
-        An instance of [Platform](utilities.md#hatch.utils.platform.Platform).
+        An instance of [Platform](../utilities.md#hatch.utils.platform.Platform).
         """
         return self.__platform
 
     @property
     def data_directory(self):
         """
-        The [directory](../config/hatch.md#environments) reserved exclusively for this plugin as a path-like object.
+        The [directory](../../config/hatch.md#environments) reserved exclusively for this plugin as a path-like object.
         """
         return self.__data_directory
 
@@ -135,7 +152,12 @@ class EnvironmentInterface(ABC):
     @property
     def system_python(self):
         if self._system_python is None:
-            system_python = os.environ.get(AppEnvVars.PYTHON, sys.executable)
+            system_python = (
+                os.environ.get(AppEnvVars.PYTHON)
+                or self.platform.modules.shutil.which('python')
+                or self.platform.modules.shutil.which('python3')
+                or sys.executable
+            )
             if not isabs(system_python):
                 system_python = self.platform.modules.shutil.which(system_python)
 
@@ -169,9 +191,13 @@ class EnvironmentInterface(ABC):
                         f'Environment variable `{key}` of field `tool.hatch.envs.{self.name}.env-vars` must be a string'
                     )
 
-            env_vars = env_vars.copy()
-            env_vars[AppEnvVars.ENV_ACTIVE] = self.name
-            self._env_vars = env_vars
+            new_env_vars = {}
+            with self.metadata.context.apply_context(self.context):
+                for key, value in env_vars.items():
+                    new_env_vars[key] = self.metadata.context.format(value)
+
+            new_env_vars[AppEnvVars.ENV_ACTIVE] = self.name
+            self._env_vars = new_env_vars
 
         return self._env_vars
 
@@ -244,23 +270,24 @@ class EnvironmentInterface(ABC):
             from packaging.requirements import InvalidRequirement, Requirement
 
             dependencies_complex = []
-            for option in ('dependencies', 'extra-dependencies'):
-                dependencies = self.config.get(option, [])
-                if not isinstance(dependencies, list):
-                    raise TypeError(f'Field `tool.hatch.envs.{self.name}.{option}` must be an array')
+            with self.metadata.context.apply_context(self.context):
+                for option in ('dependencies', 'extra-dependencies'):
+                    dependencies = self.config.get(option, [])
+                    if not isinstance(dependencies, list):
+                        raise TypeError(f'Field `tool.hatch.envs.{self.name}.{option}` must be an array')
 
-                for i, entry in enumerate(dependencies, 1):
-                    if not isinstance(entry, str):
-                        raise TypeError(
-                            f'Dependency #{i} of field `tool.hatch.envs.{self.name}.{option}` must be a string'
-                        )
+                    for i, entry in enumerate(dependencies, 1):
+                        if not isinstance(entry, str):
+                            raise TypeError(
+                                f'Dependency #{i} of field `tool.hatch.envs.{self.name}.{option}` must be a string'
+                            )
 
-                    try:
-                        dependencies_complex.append(Requirement(entry))
-                    except InvalidRequirement as e:
-                        raise ValueError(
-                            f'Dependency #{i} of field `tool.hatch.envs.{self.name}.{option}` is invalid: {e}'
-                        )
+                        try:
+                            dependencies_complex.append(Requirement(self.metadata.context.format(entry)))
+                        except InvalidRequirement as e:
+                            raise ValueError(
+                                f'Dependency #{i} of field `tool.hatch.envs.{self.name}.{option}` is invalid: {e}'
+                            )
 
             self._environment_dependencies_complex = dependencies_complex
 
@@ -269,7 +296,7 @@ class EnvironmentInterface(ABC):
     @property
     def environment_dependencies(self) -> list[str]:
         """
-        The list of all [environment dependencies](../config/environment.md#dependencies).
+        The list of all [environment dependencies](../../config/environment/overview.md#dependencies).
         """
         if self._environment_dependencies is None:
             self._environment_dependencies = [str(dependency) for dependency in self.environment_dependencies_complex]
@@ -285,6 +312,8 @@ class EnvironmentInterface(ABC):
             # they will already be installed along with the project
             if not self.skip_install and self.dev_mode:
                 dependencies_complex.extend(self.metadata.core.dependencies_complex.values())
+                for feature in self.features:
+                    dependencies_complex.extend(self.metadata.core.optional_dependencies_complex[feature].values())
 
             self._dependencies_complex = dependencies_complex
 
@@ -293,9 +322,10 @@ class EnvironmentInterface(ABC):
     @property
     def dependencies(self) -> list[str]:
         """
-        The list of all [project dependencies](../config/metadata.md#dependencies) (if
-        [installed](../config/environment.md#skip-install) and in [dev mode](../config/environment.md#dev-mode)) and
-        [environment dependencies](../config/environment.md#dependencies).
+        The list of all [project dependencies](../../config/metadata.md#dependencies) (if
+        [installed](../../config/environment/overview.md#skip-install) and in
+        [dev mode](../../config/environment/overview.md#dev-mode)) and
+        [environment dependencies](../../config/environment/overview.md#dependencies).
         """
         if self._dependencies is None:
             self._dependencies = [str(dependency) for dependency in self.dependencies_complex]
@@ -352,7 +382,7 @@ class EnvironmentInterface(ABC):
             ```
         """
         if self._skip_install is None:
-            skip_install = self.config.get('skip-install', False)
+            skip_install = self.config.get('skip-install', not self.metadata.has_project_file())
             if not isinstance(skip_install, bool):
                 raise TypeError(f'Field `tool.hatch.envs.{self.name}.skip-install` must be a boolean')
 
@@ -562,8 +592,8 @@ class EnvironmentInterface(ABC):
         :material-align-horizontal-left: **REQUIRED** :material-align-horizontal-right:
 
         This should perform the necessary steps to completely remove the environment from the system and will only
-        be triggered manually by users with the [`env remove`](../cli/reference.md#hatch-env-remove) or
-        [`env prune`](../cli/reference.md#hatch-env-prune) commands.
+        be triggered manually by users with the [`env remove`](../../cli/reference.md#hatch-env-remove) or
+        [`env prune`](../../cli/reference.md#hatch-env-prune) commands.
         """
 
     @abstractmethod
@@ -597,7 +627,7 @@ class EnvironmentInterface(ABC):
         :material-align-horizontal-left: **REQUIRED** :material-align-horizontal-right:
 
         This should indicate whether or not the environment is compatible with the current
-        [dependencies](environment.md#hatch.env.plugin.interface.EnvironmentInterface.dependencies).
+        [dependencies](reference.md#hatch.env.plugin.interface.EnvironmentInterface.dependencies).
         """
 
     @abstractmethod
@@ -606,14 +636,14 @@ class EnvironmentInterface(ABC):
         :material-align-horizontal-left: **REQUIRED** :material-align-horizontal-right:
 
         This should install the
-        [dependencies](environment.md#hatch.env.plugin.interface.EnvironmentInterface.dependencies)
+        [dependencies](reference.md#hatch.env.plugin.interface.EnvironmentInterface.dependencies)
         in the environment.
         """
 
     @contextmanager
     def build_environment(self, dependencies: list[str]):
         """
-        This should set up an isolated environment in which to [`build`](../cli/reference.md#hatch-build) the project
+        This should set up an isolated environment in which to [`build`](../../cli/reference.md#hatch-build) the project
         given a set of dependencies and must be a context manager:
 
         ```python
@@ -622,9 +652,9 @@ class EnvironmentInterface(ABC):
         ```
 
         The build environment should reflect any
-        [environment variables](environment.md#hatch.env.plugin.interface.EnvironmentInterface.get_env_vars)
+        [environment variables](reference.md#hatch.env.plugin.interface.EnvironmentInterface.get_env_vars)
         the user defined either currently or at the time of
-        [creation](environment.md#hatch.env.plugin.interface.EnvironmentInterface.create).
+        [creation](reference.md#hatch.env.plugin.interface.EnvironmentInterface.create).
         """
         with self.get_env_vars():
             yield
@@ -632,7 +662,7 @@ class EnvironmentInterface(ABC):
     def get_build_process(self, build_environment, **kwargs):
         """
         This will be called when the
-        [build environment](environment.md#hatch.env.plugin.interface.EnvironmentInterface.build_environment)
+        [build environment](reference.md#hatch.env.plugin.interface.EnvironmentInterface.build_environment)
         is active:
 
         ```python
@@ -643,29 +673,29 @@ class EnvironmentInterface(ABC):
         This should return the standard library's
         [subprocess.Popen](https://docs.python.org/3/library/subprocess.html#subprocess.Popen)
         with all output captured by `stdout`. The command is constructed by passing all keyword arguments to
-        [construct_build_command](environment.md#hatch.env.plugin.interface.EnvironmentInterface.construct_build_command).
+        [construct_build_command](reference.md#hatch.env.plugin.interface.EnvironmentInterface.construct_build_command).
 
         For an example, open the default implementation below:
         """
         return self.platform.capture_process(self.construct_build_command(**kwargs))
 
-    def enter_shell(self, name, path):
+    def enter_shell(self, name, path, args):
         """
-        Spawn a [shell](../config/hatch.md#shell) within the environment.
+        Spawn a [shell](../../config/hatch.md#shell) within the environment.
 
         This should either use
-        [command_context](environment.md#hatch.env.plugin.interface.EnvironmentInterface.command_context)
+        [command_context](reference.md#hatch.env.plugin.interface.EnvironmentInterface.command_context)
         directly or provide the same guarantee.
         """
         with self.command_context():
-            self.platform.exit_with_command([path])
+            self.platform.exit_with_command([path, *args])
 
     def run_shell_command(self, command: str):
         """
         This should return the standard library's
         [subprocess.CompletedProcess](https://docs.python.org/3/library/subprocess.html#subprocess.CompletedProcess)
         and will always be called when the
-        [command_context](environment.md#hatch.env.plugin.interface.EnvironmentInterface.command_context)
+        [command_context](reference.md#hatch.env.plugin.interface.EnvironmentInterface.command_context)
         is active, with the expectation of providing the same guarantee.
         """
         return self.platform.run_command(command, shell=True)
@@ -674,9 +704,9 @@ class EnvironmentInterface(ABC):
     def command_context(self):
         """
         A context manager that when active should make executed shell commands reflect any
-        [environment variables](environment.md#hatch.env.plugin.interface.EnvironmentInterface.get_env_vars)
+        [environment variables](reference.md#hatch.env.plugin.interface.EnvironmentInterface.get_env_vars)
         the user defined either currently or at the time of
-        [creation](environment.md#hatch.env.plugin.interface.EnvironmentInterface.create).
+        [creation](reference.md#hatch.env.plugin.interface.EnvironmentInterface.create).
 
         For an example, open the default implementation below:
         """
@@ -686,23 +716,24 @@ class EnvironmentInterface(ABC):
     def resolve_commands(self, commands: list[str]):
         """
         This expands each command into one or more commands based on any
-        [scripts](../config/environment.md#scripts) that the user defined.
+        [scripts](../../config/environment/overview.md#scripts) that the user defined.
         """
         for command in commands:
             yield from self.expand_command(command)
 
     def expand_command(self, command):
-        possible_script, _, remaining = command.partition(' ')
+        possible_script, args, ignore_exit_code = parse_script_command(command)
 
         # Indicate undefined
-        if not remaining:
-            remaining = None
+        if not args:
+            args = None
 
-        if possible_script in self.scripts:
-            for cmd in self.scripts[possible_script]:
-                yield self.metadata.context.format(cmd, args=remaining).strip()
-        else:
-            yield self.metadata.context.format(command, args=remaining).strip()
+        with self.metadata.context.apply_context(self.context):
+            if possible_script in self.scripts:
+                for cmd in self.scripts[possible_script]:
+                    yield self.metadata.context.format(cmd, args=args).strip()
+            else:
+                yield self.metadata.context.format(command, args=args).strip()
 
     def construct_build_command(
         self,
@@ -716,8 +747,8 @@ class EnvironmentInterface(ABC):
         clean_only=False,
     ):
         """
-        This is the canonical way [`build`](../cli/reference.md#hatch-build) command options are translated to
-        a subprocess command issued to [builders](builder.md).
+        This is the canonical way [`build`](../../cli/reference.md#hatch-build) command options are translated to
+        a subprocess command issued to [builders](../builder/reference.md).
         """
         command = ['python', '-u', '-m', 'hatchling', 'build', '--app']
 
@@ -745,36 +776,29 @@ class EnvironmentInterface(ABC):
 
         return command
 
-    def construct_pip_install_command(self, args: list[str], verbosity=None):
+    def construct_pip_install_command(self, args: list[str]):
         """
         A convenience method for constructing a [`pip install`](https://pip.pypa.io/en/stable/cli/pip_install/)
         command with the given verbosity. The default verbosity is set to one less than Hatch's verbosity.
         """
-        if verbosity is None:
-            # Default to -1 verbosity
-            verbosity = self.verbosity - 1
-
         command = ['python', '-u', '-m', 'pip', 'install', '--disable-pip-version-check', '--no-python-version-warning']
 
-        if verbosity < 0:
-            command.append(f"-{'q' * abs(verbosity)}")
-        elif verbosity > 0:
-            command.append(f"-{'v' * abs(verbosity)}")
+        # Default to -1 verbosity
+        add_verbosity_flag(command, self.verbosity, adjustment=-1)
 
         command.extend(args)
-
         return command
 
     def join_command_args(self, args: list[str]):
         """
-        This is used by the [`run`](../cli/reference.md#hatch-run) command to construct the root command string
+        This is used by the [`run`](../../cli/reference.md#hatch-run) command to construct the root command string
         from the received arguments.
         """
         return self.platform.join_command_args(args)
 
     def apply_features(self, requirement: str):
         """
-        A convenience method that applies any user defined [features](../config/environment.md#features)
+        A convenience method that applies any user defined [features](../../config/environment/overview.md#features)
         to the given requirement.
         """
         if self.features:
@@ -785,8 +809,8 @@ class EnvironmentInterface(ABC):
 
     def check_compatibility(self):
         """
-        This raises an exception if the environment is not compatible with the user's setup.
-        The default behavior checks for [platform compatibility](../config/environment.md#supported-platforms)
+        This raises an exception if the environment is not compatible with the user's setup. The default behavior
+        checks for [platform compatibility](../../config/environment/overview.md#supported-platforms)
         and any method override should keep this check.
         """
         if self.platforms and self.platform.name not in self.platforms:
@@ -802,20 +826,29 @@ class EnvironmentInterface(ABC):
         """
         return EnvVars(self.env_vars, self.env_include, self.env_exclude)
 
-    @staticmethod
-    def get_option_types() -> dict:
-        """
-        Returns a mapping of supported options to their respective types so that they can be used by
-        [overrides](../config/environment.md#option-overrides).
-        """
-        return {}
-
     def get_env_var_option(self, option: str) -> str:
         """
         Returns the value of the upper-cased environment variable `HATCH_ENV_TYPE_<PLUGIN_NAME>_<option>`.
         """
         return os.environ.get(f'{AppEnvVars.ENV_OPTION_PREFIX}{self.PLUGIN_NAME}_{option}'.upper(), '')
 
+    def get_context(self):
+        """
+        Returns a subclass of
+        [EnvironmentContextFormatter](../utilities.md#hatch.env.context.EnvironmentContextFormatter).
+        """
+        from hatch.env.context import EnvironmentContextFormatter
+
+        return EnvironmentContextFormatter(self)
+
+    @staticmethod
+    def get_option_types() -> dict:
+        """
+        Returns a mapping of supported options to their respective types so that they can be used by
+        [overrides](../../config/environment/advanced.md#option-overrides).
+        """
+        return {}
+
     def __enter__(self):
         self.activate()
         return self
@@ -838,14 +871,16 @@ def expand_script_commands(env_name, script_name, commands, config, seen, active
     expanded_commands = []
 
     for command in commands:
-        possible_script, _, remaining = command.partition(' ')
+        possible_script, args, ignore_exit_code = parse_script_command(command)
 
         if possible_script in config:
-            cmds = expand_script_commands(env_name, possible_script, config[possible_script], config, seen, active)
-            if remaining:
-                expanded_commands.extend(f'{cmd} {remaining}' for cmd in cmds)
-            else:
-                expanded_commands.extend(cmds)
+            expanded_commands.extend(
+                format_script_commands(
+                    expand_script_commands(env_name, possible_script, config[possible_script], config, seen, active),
+                    args,
+                    ignore_exit_code,
+                )
+            )
         else:
             expanded_commands.append(command)
 
