@@ -93,26 +93,26 @@ class TestDynamic:
         assert metadata.core.dynamic == ['version']
 
 
-class TestName:
+class TestRawName:
     def test_dynamic(self, isolation):
         metadata = ProjectMetadata(str(isolation), None, {'project': {'name': 9000, 'dynamic': ['name']}})
 
         with pytest.raises(
             ValueError, match='Static metadata field `name` cannot be present in field `project.dynamic`'
         ):
-            _ = metadata.core.name
+            _ = metadata.core.raw_name
 
     def test_missing(self, isolation):
         metadata = ProjectMetadata(str(isolation), None, {'project': {}})
 
         with pytest.raises(ValueError, match='Missing required field `project.name`'):
-            _ = metadata.core.name
+            _ = metadata.core.raw_name
 
     def test_not_string(self, isolation):
         metadata = ProjectMetadata(str(isolation), None, {'project': {'name': 9000}})
 
         with pytest.raises(TypeError, match='Field `project.name` must be a string'):
-            _ = metadata.core.name
+            _ = metadata.core.raw_name
 
     def test_invalid(self, isolation):
         metadata = ProjectMetadata(str(isolation), None, {'project': {'name': 'my app'}})
@@ -124,9 +124,17 @@ class TestName:
                 'hyphens, and periods, and must begin and end with ASCII letters/digits.'
             ),
         ):
-            _ = metadata.core.name
+            _ = metadata.core.raw_name
 
-    @pytest.mark.parametrize('name', ['my--app', 'my__app', 'my..app'])
+    def test_correct(self, isolation):
+        name = 'My.App'
+        metadata = ProjectMetadata(str(isolation), None, {'project': {'name': name}})
+
+        assert metadata.core.raw_name is metadata.core.raw_name is name
+
+
+class TestName:
+    @pytest.mark.parametrize('name', ['My--App', 'My__App', 'My..App'])
     def test_normalization(self, isolation, name):
         metadata = ProjectMetadata(str(isolation), None, {'project': {'name': name}})
 
@@ -884,10 +892,32 @@ class TestClassifiers:
         with pytest.raises(TypeError, match='Classifier #1 of field `project.classifiers` must be a string'):
             _ = metadata.core.classifiers
 
-    def test_correct(self, isolation):
-        metadata = ProjectMetadata(str(isolation), None, {'project': {'classifiers': ['foo', 'foo', 'bar']}})
+    def test_entry_unknown(self, isolation):
+        metadata = ProjectMetadata(str(isolation), None, {'project': {'classifiers': ['foo']}})
 
-        assert metadata.core.classifiers == metadata.core.classifiers == ['bar', 'foo']
+        with pytest.raises(ValueError, match='Unknown classifier in field `project.classifiers`: foo'):
+            _ = metadata.core.classifiers
+
+    def test_correct(self, isolation):
+        classifiers = [
+            'Programming Language :: Python :: 3.11',
+            'Programming Language :: Python :: 3.11',
+            'Programming Language :: Python :: 3.9',
+            'Development Status :: 4 - Beta',
+            'Private :: Do Not Upload',
+        ]
+        metadata = ProjectMetadata(str(isolation), None, {'project': {'classifiers': classifiers}})
+
+        assert (
+            metadata.core.classifiers
+            == metadata.core.classifiers
+            == [
+                'Private :: Do Not Upload',
+                'Development Status :: 4 - Beta',
+                'Programming Language :: Python :: 3.9',
+                'Programming Language :: Python :: 3.11',
+            ]
+        )
 
 
 class TestURLs:
@@ -1084,7 +1114,11 @@ class TestDependencies:
         )
 
         with pytest.raises(
-            ValueError, match='Dependency #1 of field `project.dependencies` cannot be a direct reference'
+            ValueError,
+            match=(
+                'Dependency #1 of field `project.dependencies` cannot be a direct reference unless '
+                'field `tool.hatch.metadata.allow-direct-references` is set to `true`'
+            ),
         ):
             _ = metadata.core.dependencies
 
@@ -1222,7 +1256,10 @@ class TestOptionalDependencies:
 
         with pytest.raises(
             ValueError,
-            match='Dependency #1 of option `foo` of field `project.optional-dependencies` cannot be a direct reference',
+            match=(
+                'Dependency #1 of option `foo` of field `project.optional-dependencies` cannot be a direct reference '
+                'unless field `tool.hatch.metadata.allow-direct-references` is set to `true`'
+            ),
         ):
             _ = metadata.core.optional_dependencies
 
