@@ -14,7 +14,7 @@ def test_default(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -45,7 +45,7 @@ def test_default_as_json(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -71,7 +71,7 @@ def test_single_only(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -110,7 +110,7 @@ def test_single_and_matrix(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -153,7 +153,7 @@ def test_default_matrix_only(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -192,7 +192,7 @@ def test_all_matrix_types_with_single(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -244,7 +244,7 @@ def test_specific(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -281,7 +281,7 @@ def test_specific_unknown(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -307,7 +307,7 @@ def test_optional_columns(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -318,7 +318,8 @@ def test_optional_columns(hatch, helpers, temp_dir, config_file):
     data_path = temp_dir / 'data'
     data_path.mkdir()
 
-    dependencies = ['python___dateutil', 'bAr.Baz[TLS]   >=1.2RC5', 'Foo;python_version<"3.8"']
+    dependencies = ['python___dateutil', 'bAr.Baz[TLS]   >=1.2RC5']
+    extra_dependencies = ['Foo;python_version<"3.8"']
     env_vars = {'FOO': '1', 'BAR': '2'}
     description = """
 Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna \
@@ -334,13 +335,21 @@ occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim
             'matrix': [{'version': ['9000', '3.14'], 'py': ['39', '310']}],
             'description': description,
             'dependencies': dependencies,
+            'extra-dependencies': extra_dependencies,
             'env-vars': env_vars,
             'features': ['Foo...Bar', 'Baz', 'baZ'],
             'scripts': {'test': 'pytest', 'build': 'python -m build', '_foo': 'test'},
         },
     )
     helpers.update_project_environment(
-        project, 'foo', {'description': description, 'dependencies': dependencies, 'env-vars': env_vars}
+        project,
+        'foo',
+        {
+            'description': description,
+            'dependencies': dependencies,
+            'extra-dependencies': extra_dependencies,
+            'env-vars': env_vars,
+        },
     )
 
     with project_path.as_cwd():
