@@ -7,11 +7,21 @@ from hatch.project.core import Project
 from hatch.utils.structures import EnvVars
 
 
+def test_help(hatch):
+    short = hatch('run', '-h')
+    assert short.exit_code == 0, short.output
+
+    long = hatch('run', '--help')
+    assert long.exit_code == 0, long.output
+
+    assert short.output == long.output
+
+
 def test_automatic_creation(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -32,6 +42,7 @@ def test_automatic_creation(hatch, helpers, temp_dir, config_file):
     assert result.output == helpers.dedent(
         """
         Creating environment: default
+        Checking dependencies
         """
     )
     output_file = project_path / 'test.txt'
@@ -65,7 +76,7 @@ def test_enter_project_directory(hatch, config_file, helpers, temp_dir):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -92,6 +103,7 @@ def test_enter_project_directory(hatch, config_file, helpers, temp_dir):
     assert result.output == helpers.dedent(
         """
         Creating environment: default
+        Checking dependencies
         """
     )
     output_file = project_path / 'test.txt'
@@ -126,7 +138,7 @@ def test_sync_dependencies(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -147,6 +159,7 @@ def test_sync_dependencies(hatch, helpers, temp_dir, config_file):
     assert result.output == helpers.dedent(
         """
         Creating environment: default
+        Checking dependencies
         """
     )
 
@@ -187,6 +200,7 @@ def test_sync_dependencies(hatch, helpers, temp_dir, config_file):
     assert result.exit_code == 0, result.output
     assert result.output == helpers.dedent(
         """
+        Checking dependencies
         Syncing dependencies
         """
     )
@@ -201,7 +215,7 @@ def test_sync_project_dependencies(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -220,6 +234,7 @@ def test_sync_project_dependencies(hatch, helpers, temp_dir, config_file):
         """
         Creating environment: default
         Installing project in development mode
+        Checking dependencies
         """
     )
 
@@ -260,6 +275,7 @@ def test_sync_project_dependencies(hatch, helpers, temp_dir, config_file):
     assert result.exit_code == 0, result.output
     assert result.output == helpers.dedent(
         """
+        Checking dependencies
         Syncing dependencies
         """
     )
@@ -274,7 +290,7 @@ def test_sync_project_features(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -299,6 +315,7 @@ def test_sync_project_features(hatch, helpers, temp_dir, config_file):
         """
         Creating environment: default
         Installing project in development mode
+        Checking dependencies
         """
     )
 
@@ -339,6 +356,7 @@ def test_sync_project_features(hatch, helpers, temp_dir, config_file):
     assert result.exit_code == 0, result.output
     assert result.output == helpers.dedent(
         """
+        Checking dependencies
         Syncing dependencies
         """
     )
@@ -352,7 +370,7 @@ def test_scripts(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -377,6 +395,7 @@ def test_scripts(hatch, helpers, temp_dir, config_file):
     assert result.output == helpers.dedent(
         """
         Creating environment: default
+        Checking dependencies
         """
     )
     output_file = project_path / 'test.txt'
@@ -410,7 +429,7 @@ def test_scripts_specific_environment(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -441,6 +460,7 @@ def test_scripts_specific_environment(hatch, helpers, temp_dir, config_file):
     assert result.output == helpers.dedent(
         """
         Creating environment: test
+        Checking dependencies
         """
     )
     output_file = project_path / 'test.txt'
@@ -473,11 +493,11 @@ def test_scripts_specific_environment(hatch, helpers, temp_dir, config_file):
 
 
 @pytest.mark.requires_internet
-def test_scripts_no_environment(hatch, temp_dir, config_file):
+def test_scripts_no_environment(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -497,7 +517,12 @@ def test_scripts_no_environment(hatch, temp_dir, config_file):
         result = hatch('run', ':py', "import pathlib,sys;pathlib.Path('test.txt').write_text(sys.executable)")
 
     assert result.exit_code == 0, result.output
-    assert not result.output
+    assert result.output == helpers.dedent(
+        """
+        Checking dependencies
+        """
+    )
+
     output_file = project_path / 'test.txt'
     assert output_file.is_file()
 
@@ -511,7 +536,7 @@ def test_error(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -545,6 +570,7 @@ def test_error(hatch, helpers, temp_dir, config_file):
     assert result.output == helpers.dedent(
         """
         Creating environment: default
+        Checking dependencies
         cmd [1] | python -c "import sys;sys.exit(3)"
         """
     )
@@ -556,7 +582,7 @@ def test_ignore_error(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -590,6 +616,7 @@ def test_ignore_error(hatch, helpers, temp_dir, config_file):
     assert result.output == helpers.dedent(
         """
         Creating environment: default
+        Checking dependencies
         cmd [1] | - python -c "import sys;sys.exit(3)"
         cmd [2] | python -c "import pathlib,sys;pathlib.Path('test.txt').write_text(sys.executable)"
         """
@@ -625,7 +652,7 @@ def test_command_expansion_error(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -650,6 +677,7 @@ def test_command_expansion_error(hatch, helpers, temp_dir, config_file):
     assert result.output == helpers.dedent(
         """
         Creating environment: default
+        Checking dependencies
         Nonexistent environment variable must set a default: FOOBAR
         """
     )
@@ -661,7 +689,7 @@ def test_verbosity(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -697,6 +725,7 @@ def test_verbosity(hatch, helpers, temp_dir, config_file):
         """
         ─────────────────────────────────── default ────────────────────────────────────
         Creating environment: default
+        Checking dependencies
         cmd [1] | python -c "import pathlib,sys;pathlib.Path('test1.txt').write_text(sys.executable)"
         cmd [2] | python -c "import pathlib,sys;pathlib.Path('test2.txt').write_text(sys.executable)"
         cmd [3] | python -c "import pathlib,sys;pathlib.Path('test3.txt').write_text(sys.executable)"
@@ -737,7 +766,7 @@ def test_matrix_no_environments(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -769,7 +798,7 @@ def test_matrix(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -794,8 +823,10 @@ def test_matrix(hatch, helpers, temp_dir, config_file):
         """
         ────────────────────────────────── test.9000 ───────────────────────────────────
         Creating environment: test.9000
+        Checking dependencies
         ─────────────────────────────────── test.42 ────────────────────────────────────
         Creating environment: test.42
+        Checking dependencies
         """
     )
     output_file = project_path / 'test.txt'
@@ -828,7 +859,7 @@ def test_incompatible_single(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -867,7 +898,7 @@ def test_incompatible_matrix_full(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -908,7 +939,7 @@ def test_incompatible_matrix_partial(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -940,6 +971,7 @@ def test_incompatible_matrix_partial(hatch, helpers, temp_dir, config_file):
         """
         ─────────────────────────────────── test.42 ────────────────────────────────────
         Creating environment: test.42
+        Checking dependencies
 
         Skipped 1 incompatible environment:
         test.9000 -> unsupported platform
@@ -976,7 +1008,7 @@ def test_incompatible_missing_python(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -1006,6 +1038,7 @@ def test_incompatible_missing_python(hatch, helpers, temp_dir, config_file):
         f"""
         ────────────────────────────────── test.py{known_version} ─────────────────────────────────{padding}
         Creating environment: test.py{known_version}
+        Checking dependencies
 
         Skipped 1 incompatible environment:
         test.py9000 -> cannot locate Python: 9000
@@ -1042,7 +1075,7 @@ def test_env_detection(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -1064,6 +1097,7 @@ def test_env_detection(hatch, helpers, temp_dir, config_file):
     assert result.output == helpers.dedent(
         """
         Creating environment: default
+        Checking dependencies
         """
     )
 
@@ -1074,6 +1108,7 @@ def test_env_detection(hatch, helpers, temp_dir, config_file):
     assert result.output == helpers.dedent(
         """
         Creating environment: foo
+        Checking dependencies
         """
     )
 
@@ -1113,7 +1148,7 @@ def test_env_detection_override(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -1135,6 +1170,7 @@ def test_env_detection_override(hatch, helpers, temp_dir, config_file):
     assert result.output == helpers.dedent(
         """
         Creating environment: default
+        Checking dependencies
         """
     )
 
@@ -1145,6 +1181,7 @@ def test_env_detection_override(hatch, helpers, temp_dir, config_file):
     assert result.output == helpers.dedent(
         """
         Creating environment: foo
+        Checking dependencies
         """
     )
 
@@ -1186,7 +1223,7 @@ def test_matrix_variable_selection_no_command(hatch, helpers, temp_dir, config_f
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -1216,7 +1253,7 @@ def test_matrix_variable_selection_duplicate_inclusion(hatch, helpers, temp_dir,
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -1253,7 +1290,7 @@ def test_matrix_variable_selection_duplicate_exclusion(hatch, helpers, temp_dir,
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -1290,7 +1327,7 @@ def test_matrix_variable_selection_python_alias(hatch, helpers, temp_dir, config
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -1327,7 +1364,7 @@ def test_matrix_variable_selection_not_matrix(hatch, helpers, temp_dir, config_f
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -1363,7 +1400,7 @@ def test_matrix_variable_selection_inclusion(hatch, helpers, temp_dir, config_fi
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -1392,6 +1429,7 @@ def test_matrix_variable_selection_inclusion(hatch, helpers, temp_dir, config_fi
         """
         ────────────────────────────────── test.9000 ───────────────────────────────────
         Creating environment: test.9000
+        Checking dependencies
         """
     )
     output_file = project_path / 'test.txt'
@@ -1425,7 +1463,7 @@ def test_matrix_variable_selection_exclusion(hatch, helpers, temp_dir, config_fi
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -1454,6 +1492,7 @@ def test_matrix_variable_selection_exclusion(hatch, helpers, temp_dir, config_fi
         """
         ─────────────────────────────────── test.42 ────────────────────────────────────
         Creating environment: test.42
+        Checking dependencies
         """
     )
     output_file = project_path / 'test.txt'
@@ -1487,7 +1526,7 @@ def test_matrix_variable_selection_exclude_all(hatch, helpers, temp_dir, config_
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -1523,7 +1562,7 @@ def test_matrix_variable_selection_include_none(hatch, helpers, temp_dir, config
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -1559,7 +1598,7 @@ def test_context_formatting_recursion(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
 
-    project_name = 'My App'
+    project_name = 'My.App'
 
     with temp_dir.as_cwd():
         result = hatch('new', project_name)
@@ -1593,6 +1632,7 @@ def test_context_formatting_recursion(hatch, helpers, temp_dir, config_file):
     assert result.output == helpers.dedent(
         """
         Creating environment: default
+        Checking dependencies
         """
     )
     output_file = project_path / 'test.txt'
